package com.qmth.themis.business.util;

import com.alibaba.fastjson.JSONObject;
import com.qmth.themis.business.constant.SystemConstant;
import org.apache.http.protocol.HTTP;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.xml.bind.DatatypeConverter;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.util.TreeMap;

/**
 * @Description: 腾讯云视频回调
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/3/30
 */
public class TencentCloudAPITC3Util {
    private final static Logger log = LoggerFactory.getLogger(TencentCloudAPITC3Util.class);

    public final static String DATE_PATTERN = "yyyy-MM-dd";
    public final static String UTC = "UTC";
    private final static String TC3 = "TC3";
    private final static String TC3_REQUEST = "tc3_request";
    private final static String TC3_HMAC_SHA256 = "TC3-HMAC-SHA256";
    private final static String SIGNED_HEADERS = "content-type;host";
    private final static Charset UTF8 = StandardCharsets.UTF_8;
    //    private final static String SECRET_ID = "AKIDz8krbsJ5yKBZQpn74WFkmLPx3*******";
//    private final static String SECRET_KEY = "Gu5t9xGARNpq86cd98joQYCN3*******";
    private final static String CT_JSON = "application/json";
    private final static String HMACSHA256 = "HmacSHA256";
    private final static String SHA256 = "SHA-256";
    private final static String VERSION = "2018-07-17";

    /**
     * hmac256算法
     *
     * @param key
     * @param msg
     * @return
     * @throws Exception
     */
    public static byte[] hmac256(byte[] key, String msg) throws Exception {
        Mac mac = Mac.getInstance(HMACSHA256);
        SecretKeySpec secretKeySpec = new SecretKeySpec(key, mac.getAlgorithm());
        mac.init(secretKeySpec);
        return mac.doFinal(msg.getBytes(UTF8));
    }

    /**
     * sha256Hex 算法
     *
     * @param input
     * @return
     * @throws Exception
     */
    public static String sha256Hex(String input) throws Exception {
        MessageDigest md = MessageDigest.getInstance(SHA256);
        byte[] d = md.digest(input.getBytes(UTF8));
        return DatatypeConverter.printHexBinary(d).toLowerCase();
    }

    /**
     * 拼接规范请求串
     *
     * @param requestMethod
     * @param offset
     * @param limit
     * @param subAppId
     * @param host
     * @return
     * @throws Exception
     */
    public static String requestStr(String requestMethod, Long offset, Long limit, Long subAppId, String host) throws Exception {
        String canonicalUri = "/";
        String canonicalQueryString = "";
        String canonicalHeaders = "content-type:application/json; charset=utf-8\n" + "host:" + host + "\n";
        String signedHeaders = "content-type;host";

        JSONObject jsonObject = new JSONObject();
        jsonObject.put(SystemConstant.LIMIT, limit);
        jsonObject.put(SystemConstant.OFFSET, offset);
        jsonObject.put(SystemConstant.SUB_APP_ID, subAppId);
//        String payload = "{\"Limit\": " + limit + ", \"Offset\": " + offset + ", \"SubAppId\": " + subAppId + "}";
        String hashedRequestPayload = sha256Hex(jsonObject.toJSONString());
        String canonicalRequest = requestMethod + "\n" + canonicalUri + "\n" + canonicalQueryString + "\n"
                + canonicalHeaders + "\n" + signedHeaders + "\n" + hashedRequestPayload;
        log.info("canonicalRequest:{}", canonicalRequest);
        return canonicalRequest;
    }

    /**
     * 拼接待签名字符串
     *
     * @param date
     * @param timestamp
     * @param service
     * @param canonicalRequest
     * @return
     * @throws Exception
     */
    public static String signStr(String date, String timestamp, String service, String canonicalRequest) throws Exception {
        String credentialScope = date + "/" + service + "/" + TC3_REQUEST;
        String hashedCanonicalRequest = sha256Hex(canonicalRequest);
        String stringToSign = TC3_HMAC_SHA256 + "\n" + timestamp + "\n" + credentialScope + "\n" + hashedCanonicalRequest;
        log.info("stringToSign:{}", stringToSign);
        return stringToSign;
    }

    /**
     * 计算签名
     *
     * @param secretKey
     * @param date
     * @param service
     * @param stringToSign
     * @return
     * @throws Exception
     */
    public static String calculateSign(String secretKey, String date, String service, String stringToSign) throws Exception {
        byte[] secretDate = hmac256((TC3 + secretKey).getBytes(UTF8), date);
        byte[] secretService = hmac256(secretDate, service);
        byte[] secretSigning = hmac256(secretService, TC3_REQUEST);
        String signature = DatatypeConverter.printHexBinary(hmac256(secretSigning, stringToSign)).toLowerCase();
        log.info("signature:{}", signature);
        return signature;
    }

    /**
     * 拼接参数
     *
     * @param secretId
     * @param date
     * @param timestamp
     * @param service
     * @param signature
     * @param action
     * @param host
     * @param subAppId
     * @return
     */
    public static TreeMap<String, String> jointParam(String secretId,
                                                     String date,
                                                     String timestamp,
                                                     String service,
                                                     String signature,
                                                     String action,
                                                     String host,
                                                     Long subAppId) {
        String credentialScope = date + "/" + service + "/" + TC3_REQUEST;
        String authorization = TC3_HMAC_SHA256 + " " + "Credential=" + secretId + "/" + credentialScope + ", "
                + "SignedHeaders=" + SIGNED_HEADERS + ", " + "Signature=" + signature;
        log.info("authorization:{}", authorization);

        TreeMap<String, String> headers = new TreeMap<String, String>();
        headers.put(SystemConstant.HEADER_AUTHORIZATION, authorization);
        headers.put(HTTP.CONTENT_TYPE, CT_JSON);
        headers.put(HTTP.TARGET_HOST, host);
        headers.put(SystemConstant.X_TC_ACTION, action);
        headers.put(SystemConstant.X_TC_TIMESTAMP, timestamp);
        headers.put(SystemConstant.X_TC_VERSION, VERSION);
        log.info("headers:{}", JacksonUtil.parseJson(headers));

        JSONObject jsonObject = new JSONObject();
        jsonObject.put(SystemConstant.LIMIT, 10L);
        jsonObject.put(SystemConstant.OFFSET, 0L);
        jsonObject.put(SystemConstant.SUB_APP_ID, subAppId);

        StringBuilder sb = new StringBuilder();
        sb.append("curl -X POST https://").append(host)
                .append(" -H \"Authorization: ").append(authorization).append("\"")
                .append(" -H Content-Type: application/json")
                .append(" -H \"Host: ").append(host).append("\"")
                .append(" -H \"X-TC-Action: ").append(action).append("\"")
                .append(" -H \"X-TC-Timestamp: ").append(timestamp).append("\"")
                .append(" -H \"X-TC-Version: ").append(VERSION).append("\"")
                .append(" -d '").append(jsonObject.toJSONString()).append("'");
        log.info("sb:{}", sb.toString());

//        try {
//            Process process = Runtime.getRuntime().exec(sb.toString());
//            process.getInputStream();
//            BufferedReader br = new BufferedReader(new InputStreamReader(process.getInputStream()));
//            String output;
//            StringBuffer response = new StringBuffer();
//            while ((output = br.readLine()) != null) {
//                response.append(output);
//            }
//            br.close();
//            System.out.println(response.toString());
//        } catch (IOException ioException) {
//            ioException.printStackTrace();
//        }
        return headers;
    }

//    public static void main(String[] args) throws Exception {
////        String service = "cvm";
////        String host = "cvm.tencentcloudapi.com";
////        String region = "ap-guangzhou";
//        String action = "DescribeInstances";
//        String version = "2017-03-12";
//        String algorithm = "TC3-HMAC-SHA256";
//        String timestamp = "1551113065";
//        //String timestamp = String.valueOf(System.currentTimeMillis() / 1000);
//        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
//        // 注意时区，否则容易出错
//        sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
//        String date = sdf.format(new Date(Long.valueOf(timestamp + "000")));
//
//        // ************* 步骤 1：拼接规范请求串 *************
//        String httpRequestMethod = "POST";
//        String canonicalUri = "/";
//        String canonicalQueryString = "";
//        String canonicalHeaders = "content-type:application/json; charset=utf-8\n" + "host:" + host + "\n";
//        String signedHeaders = "content-type;host";
//
//        String payload = "{\"Limit\": 1, \"Filters\": [{\"Values\": [\"\\u672a\\u547d\\u540d\"], \"Name\": \"instance-name\"}]}";
//        String hashedRequestPayload = sha256Hex(payload);
//        String canonicalRequest = httpRequestMethod + "\n" + canonicalUri + "\n" + canonicalQueryString + "\n"
//                + canonicalHeaders + "\n" + signedHeaders + "\n" + hashedRequestPayload;
//        System.out.println(canonicalRequest);
//
//        // ************* 步骤 2：拼接待签名字符串 *************
//        String credentialScope = date + "/" + service + "/" + "tc3_request";
//        String hashedCanonicalRequest = sha256Hex(canonicalRequest);
//        String stringToSign = algorithm + "\n" + timestamp + "\n" + credentialScope + "\n" + hashedCanonicalRequest;
//        System.out.println(stringToSign);
//
//        // ************* 步骤 3：计算签名 *************
//        byte[] secretDate = hmac256(("TC3" + SECRET_KEY).getBytes(UTF8), date);
//        byte[] secretService = hmac256(secretDate, service);
//        byte[] secretSigning = hmac256(secretService, "tc3_request");
//        String signature = DatatypeConverter.printHexBinary(hmac256(secretSigning, stringToSign)).toLowerCase();
//        System.out.println(signature);
//
//        // ************* 步骤 4：拼接 Authorization *************
//        String authorization = algorithm + " " + "Credential=" + SECRET_ID + "/" + credentialScope + ", "
//                + "SignedHeaders=" + signedHeaders + ", " + "Signature=" + signature;
//        System.out.println(authorization);
//
//        TreeMap<String, String> headers = new TreeMap<String, String>();
//        headers.put("Authorization", authorization);
//        headers.put("Content-Type", CT_JSON);
//        headers.put("Host", host);
//        headers.put("X-TC-Action", action);
//        headers.put("X-TC-Timestamp", timestamp);
//        headers.put("X-TC-Version", version);
////        headers.put("X-TC-Region", region);
//
//        StringBuilder sb = new StringBuilder();
//        sb.append("curl -X POST https://").append(host)
//                .append(" -H \"Authorization: ").append(authorization).append("\"")
//                .append(" -H \"Content-Type: application/json; charset=utf-8\"")
//                .append(" -H \"Host: ").append(host).append("\"")
//                .append(" -H \"X-TC-Action: ").append(action).append("\"")
//                .append(" -H \"X-TC-Timestamp: ").append(timestamp).append("\"")
//                .append(" -H \"X-TC-Version: ").append(version).append("\"")
//                .append(" -H \"X-TC-Region: ").append(region).append("\"")
//                .append(" -d '").append(payload).append("'");
//        System.out.println(sb.toString());
//    }
}
